/******************************************************************************
 *
 * CHECK_DISK.C
 *
 * Program: Disk space plugin for Nagios
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (nagios@nagios.org)
 * Copyright (c) 2000 Karl DeBisschop (kdebisschop@users.sourceforge.net)
 * Copyright (c) 2004 Guenther Mair (guenther.mair@datef.it)
 *
 * $Id: check_disk.c,v 2.0.0.0 2004/03/14 03:00:00 m700 Exp $
 *
 * Description:
 *
 * This plugin uses the statfs syscall to check the free space on a
 * filesystem.  If the percent used disk space is above <c_dfp>,  a
 * STATE_CRITICAL  is returned.  If the percent  used disk space is
 * above <w_dfp>,  a STATE_WARNING  is returned.  If the  specified
 * filesystem cannot be read,  a STATE_CRITICAL is returned,  other
 * errors with reading the output result in a STATE_UNKNOWN error.
 *
 * Notes:
 *  - taken from former check_disk.c which used /bin/df
 *     dependencies for other OSes other than Linux will probably
 *     be broken
 *
 *****************************************************************************/

#include "common.h"
#include "utils.h"
#include <sys/vfs.h>
#include <stdarg.h>

#define REVISION "$Revision: 2.0.0.0 $"
#define COPYRIGHT "2004"

int process_arguments (int, char **);
int validate_arguments (void);
int check_disk (float usp, float free_disk);
void print_help (void);
void print_usage (void);

const char *progname = "check_disk";

int w_df = -1;
int c_df = -1;
float w_dfp = -1.0;
float c_dfp = -1.0;
char *path = "";
int verbose = 0;


int
main (int argc, char **argv)
{
	int result = STATE_UNKNOWN;
	long total_kb, used_kb, free_kb;
	float usp = -1;
	struct statfs fs_status;

	if (process_arguments (argc, argv) != OK)
		usage ("Could not parse arguments\n");

	if (statfs(path, &fs_status)==0) {
		total_kb = (fs_status.f_bsize / 1024) * fs_status.f_blocks;
		free_kb = (fs_status.f_bsize / 1024) * fs_status.f_bavail;

		used_kb = total_kb - free_kb;
		usp = (used_kb  * 100) / total_kb;

		result = check_disk (usp, free_kb);

		if (verbose>0) {
			printf("total size: %d kb ", total_kb);
			printf("(%d blocks) ", fs_status.f_blocks);
			printf("used mem: %d kb ", used_kb);
			printf("(%.1f%%) ", usp);
			printf("free mem: %d kb ", free_kb);
			printf("(%d blocks) ", fs_status.f_bavail);
		}
	}
	else {
		usage("path argument must point to a disk\n");
	}

	printf ("DISK %s %.1f%% used (%d kb)\n",
		state_text (result), usp, used_kb);

	return result;
}

/* process command-line arguments */
int
process_arguments (int argc, char **argv)
{
	int c;

#ifdef HAVE_GETOPT_H
	int option_index = 0;
	static struct option long_options[] = {
		{"warning", required_argument, 0, 'w'},
		{"critical", required_argument, 0, 'c'},
		{"path", required_argument, 0, 'p'},
		{"partition", required_argument, 0, 'p'},
		{"verbose", no_argument, 0, 'v'},
		{"version", no_argument, 0, 'V'},
		{"help", no_argument, 0, 'h'},
		{"quiet", no_argument, 0, 'q'},

		{0, 0, 0, 0}
	};
#endif

	if (argc < 3)
		return ERROR;

	for (c = 1; c < argc; c++)
		if (strcmp ("-to", argv[c]) == 0)
			strcpy (argv[c], "-t");

	while (1) {
#ifdef HAVE_GETOPT_H
		c =
 			getopt_long (argc, argv, "+?Vqhvc:w:p:", long_options, &option_index);
#else
 		c = getopt (argc, argv, "+?Vqhvc:w:p:");
#endif

		if (c == -1 || c == EOF)
			break;

		switch (c) {
		case 'w':									/* warning time threshold */
			if (is_intnonneg (optarg)) {
				w_df = atoi (optarg);
				break;
			}
			else if (strpbrk (optarg, ",:") &&
							 strstr (optarg, "%") &&
							 sscanf (optarg, "%d%*[:,]%f%%", &w_df, &w_dfp) == 2) {
				break;
			}
			else if (strstr (optarg, "%") && sscanf (optarg, "%f%%", &w_dfp) == 1) {
				break;
			}
			else {
				usage ("Warning threshold must be integer or percentage!\n");
			}
		case 'c':									/* critical time threshold */
			if (is_intnonneg (optarg)) {
				c_df = atoi (optarg);
				break;
			}
			else if (strpbrk (optarg, ",:") &&
							 strstr (optarg, "%") &&
							 sscanf (optarg, "%d%*[,:]%f%%", &c_df, &c_dfp) == 2) {
				break;
			}
			else if (strstr (optarg, "%") && sscanf (optarg, "%f%%", &c_dfp) == 1) {
				break;
			}
			else {
				usage ("Critical threshold must be integer or percentage!\n");
			}
		case 'p':									/* path or partition */
			path = optarg;
			break;
		case 'v':									/* verbose */
			verbose++;
			break;
		case 'q':									/* verbose */
			verbose--;
			break;
		case 'V':									/* version */
			print_revision (progname, REVISION);
			exit (STATE_OK);
		case 'h':									/* help */
		case '?':									/* help */
			print_help ();
			exit (STATE_OK);
		}
	}

	c = optind;
	if (w_dfp == -1 && argc > c && is_intnonneg (argv[c]))
		w_dfp = (100.0 - atof (argv[c++]));

	if (c_dfp == -1 && argc > c && is_intnonneg (argv[c]))
		c_dfp = (100.0 - atof (argv[c++]));

	if (argc > c && strlen (path) == 0)
		path = argv[c++];

	return validate_arguments ();
}

int
validate_arguments ()
{
	if (w_df < 0 && c_df < 0 && w_dfp < 0 && c_dfp < 0) {
		printf ("INPUT ERROR: Unable to parse command line\n");
		return ERROR;
	}
	else if ((w_dfp >= 0 || c_dfp >= 0)
					 && (w_dfp < 0 || c_dfp < 0 || w_dfp > 100 || c_dfp > 100
							 || c_dfp > w_dfp)) {
		printf
			("INPUT ERROR: C_DFP (%f) should be less than W_DFP (%f) and both should be between zero and 100 percent, inclusive\n",
			 c_dfp, w_dfp);
		return ERROR;
	}
	else if ((w_df > 0 || c_df > 0) && (w_df < 0 || c_df < 0 || c_df > w_df)) {
		printf
			("INPUT ERROR: C_DF (%d) should be less than W_DF (%d) and both should be greater than zero\n",
			 c_df, w_df);
		return ERROR;
	}
	else if (path == NULL) {
		printf("INPUT ERROR: path must be set\n");
		return ERROR;
	}
	else {
		return OK;
	}
}

int
check_disk (float usp, float free_disk)
{
	int result = STATE_UNKNOWN;
	if (verbose>0) {
		printf("USP: %.1f%% ", usp);
		printf("C_DFP: %.1f%% ", c_dfp);
		printf("C_DF: %d ", c_df);
		printf("W_DFP: %.1f%% ", w_dfp);
		printf("W_DF: %d ", w_df);
	}
	/* check the percent used space against tresholds */
	if (usp >= 0 && usp >= (100.0 - c_dfp))
		result = STATE_CRITICAL;
	else if (c_df >= 0 && free_disk <= c_df)
		result = STATE_CRITICAL;
	else if (usp >= 0 && usp >= (100.0 - w_dfp))
		result = STATE_WARNING;
	else if (w_df >= 0 && free_disk <= w_df)
		result = STATE_WARNING;
	else if (usp >= 0.0)
		result = STATE_OK;
	return result;
}

void
print_help (void)
{
	print_revision (progname, REVISION);
	printf
		("Copyright (c) 2004 Ethan Galstad/Karl DeBisschop/Guenther Mair\n\n"
		 "This plugin will check the percent of used disk space on a file\n"
		 "system and generate an alert if percentage is above one of the\n"
		 "threshold values.\n\n");
	print_usage ();
	printf
		("\nOptions:\n"
		 " -w, --warning=INTEGER\n"
		 "   Exit with WARNING status if less than INTEGER kilobytes of disk are free\n"
		 " -w, --warning=PERCENT%%\n"
		 "   Exit with WARNING status if less than PERCENT of disk space is free\n"
		 " -c, --critical=INTEGER\n"
		 "   Exit with CRITICAL status if less than INTEGER kilobytes of disk are free\n"
		 " -c, --critical=PERCENT%%\n"
		 "   Exit with CRITCAL status if less than PERCENT of disk space is free\n"
		 " -p, --path=PATH, --partition=PARTTION\n"
		 "    Path or partition (checks all mounted partitions if unspecified)\n"
		 " -v, --verbose\n"
		 "    Show details for command-line debugging (do not use with nagios server)\n"
		 " -h, --help\n"
		 "    Print detailed help screen\n"
		 " -V, --version\n" "    Print version information\n\n");
	support ();
}

void
print_usage (void)
{
	printf
		("Usage: %s -w limit -c limit -p path [--verbose]\n"
		 "       %s (-h|--help)\n"
		 "       %s (-V|--version)\n", progname, progname, progname);
}
